﻿using System;
using System.Collections.Generic;
using System.Linq;
using GameOn.Web.Entities;
using GameOn.Web.Models;

namespace GameOn.Web.Services
{
    /// <summary>
    /// A read-only service for retrieving players
    /// </summary>
    public class PlayerService : IPlayerService
    {
        private GameOnEntities Entities { get; set; }

        public PlayerService(GameOnEntities entities)
        {
            Entities = entities;
        }

        /// <summary>
        /// Gets a single player by ID
        /// </summary>
        /// <param name="id"></param>
        /// <returns></returns>
        public Player GetPlayer(int id)
        {
            return Entities.Players.FirstOrDefault(p => p.Id == id);
        }

        /// <summary>
        /// Returns a ranked list of players, sorted by their ELO rating.
        /// </summary>
        /// <returns></returns>
        public List<Player> GetPlayersByRank()
        {
            return Entities.Players.OrderByDescending(p => p.CurrentRank).ToList();
        }

        public Player GetTopRankingPlayer()
        {
            return Entities.Players.OrderByDescending(p => p.CurrentRank).FirstOrDefault();
        }

        private IQueryable<PlayerMatchesSummary> GetPlayerMatchesSummaryQuery()
        {
            var query = from p in Entities.Players
                        select
                            new PlayerMatchesSummary
                            {
                                Player = p,
                                HighestRanking = p.RankHistory.Max(rh => rh.Rank),
                                LowestRanking = p.RankHistory.Min(rh => rh.Rank),
                                Won = p.MatchesWon.Count(),
                                Played = p.MatchesAsPlayer1.Count() + p.MatchesAsPlayer2.Count()
                            };
            return query;
        }

        /// <summary>
        /// Gets a summary of all of the matches that a Player has played
        /// </summary>
        public PlayerMatchesSummary GetPlayerMatchesSummary(int playerId)
        {
            var query = GetPlayerMatchesSummaryQuery();
            return query.FirstOrDefault(p => p.Player.Id == playerId);
        }

        /// <summary>
        /// Gets a list of players and summaries of all of the matches each Player has played, sorted by rank
        /// </summary>
        public List<PlayerMatchesSummary> GetListOfPlayersWithMatchSummaries()
        {
            var query = GetPlayerMatchesSummaryQuery();
            return query.OrderByDescending(p=>p.Player.CurrentRank).ToList();
        }

        /// <summary>
        /// Gets a list of player IDs and comma-separated Rank data for drawing players' sparklines
        /// </summary>
        /// <returns>A list of tuples, each a player ID as int, and a list of Ranks as a comma-separated string</returns>
        public List<Tuple<int, string>> GetRankHistoryDataForSparkLine()
        {
            var query = from h in Entities.RankHistories 
                        orderby h.PlayerId, h.Id
                        select new {h.PlayerId, h.Rank};

            var results = new List<Tuple<int, string>>();

            int lastPlayerId = 0;
            string data = "";
            bool first = true;
            foreach (var history in query)
            {
                if (lastPlayerId != 0 && lastPlayerId != history.PlayerId)
                {
                    var result = new Tuple<int, string>(lastPlayerId, data);
                    results.Add(result);
                    data = "";
                    first = true;
                }

                if (!first)
                {
                    data += ",";
                }
                
                data += history.Rank;
                lastPlayerId = history.PlayerId;
                first = false;
            }

            if (lastPlayerId != 0)
            {
                var result = new Tuple<int, string>(lastPlayerId, data);
                results.Add(result);
            }

            return results;
        }

        /// <summary>
        /// Gets a player ID and comma-separated Rank data for drawing a player's sparkline
        /// </summary>
        /// <param name="playerId">A player ID. Returns a </param>
        /// <returns>A tuple: a player ID as int, and a list of Ranks as a comma-separated string</returns>
        public Tuple<int, string> GetRankHistoryDataForSparkLine(int playerId)
        {
            throw new NotImplementedException();
        }
    }
}
